@@ -8,8 +8,8 @@
 ;; URL: https://github.com/dimitri/switch-window
 ;;      http://tapoueh.org/emacs/switch-window.html
 ;; Git-URL: https://github.com/dimitri/switch-window.git
-;; Package-Version: 1.6.2
-;; Package-Revision: 204f9fc1a398
+;; Package-Version: 20250401.839
+;; Package-Revision: 8f771b571a1e
 ;; Created: 2010-04-30
 ;; Keywords: convenience
 ;; Licence: WTFPL, grab your copy here: http://sam.zoy.org/wtfpl/
@@ -227,7 +227,7 @@
 (require 'switch-window-mvborder)
 
 (defgroup switch-window nil
-  "switch-window customization group"
+  "Switch-window customization group."
   :group 'convenience)
 
 (defcustom switch-window-background nil
@@ -252,29 +252,31 @@
 
 (defcustom switch-window-shortcut-style 'quail
   "Use either keyboard layout or alphabet shortcut style."
-  :type '(choice (const :tag "Alphabet" 'alphabet)
-                 (const :tag "Keyboard Layout" 'quail)
-                 (const :tag "Qwerty Homekeys Layout" 'qwerty))
+  :type '(choice (const :tag "Alphabet" alphabet)
+                 (const :tag "Keyboard Layout" quail)
+                 (const :tag "Qwerty Homekeys Layout" qwerty))
   :group 'switch-window)
 
 (defcustom switch-window-qwerty-shortcuts
   '("a" "s" "d" "f" "j" "k" "l" ";" "g" "h"
     "q" "w" "e" "r" "t" "y" "u" "i" "p"
     "z" "x" "c" "v" "b" "n" "m")
-  "The list of characters used when ‘switch-window-shortcut-style’ is 'qwerty'."
-  :type 'list
+  "The list of characters used when `switch-window-shortcut-style' is \\='qwerty ."
+  :type '(repeat string)
   :group 'switch-window)
 
 (defcustom switch-window-shortcut-appearance 'text
   "Switch-window shortcut's appearance."
-  :type '(choice (const :tag "Show shortcut with text" 'text)
-                 (const :tag "Show shortcut with Ascii art." 'asciiart)
-                 (const :tag "Show shortcut with image." 'image))
+  :type '(choice (const :tag "Show shortcut with text" text)
+                 (const :tag "Show shortcut with Ascii art." asciiart)
+                 (const :tag "Show shortcut with image." image))
   :group 'switch-window)
 
-(defcustom switch-window-image-directory (locate-user-emacs-file "switch-window/image")
+(defcustom switch-window-image-directory
+  (locate-user-emacs-file "switch-window/image")
   "Switch-window image directory.
-If `switch-window-shortcut-appearance' set to 'image, image file
+
+If `switch-window-shortcut-appearance' set to \\='image, image file
 will be found in this directory."
   :type 'directory
   :group 'switch-window)
@@ -290,10 +292,10 @@ a window's label string, two optional arguments:
   :type 'function
   :group 'switch-window)
 
-(defcustom switch-window-input-style 'default
+(defcustom switch-window-input-style 'minibuffer
   "Use `read-event' or `read-from-minibuffer' to get user's input."
-  :type '(choice (const :tag "Get input by read-event" 'default)
-                 (const :tag "Get input from minibuffer" 'minibuffer))
+  :type '(choice (const :tag "Get input by read-event" read-event)
+                 (const :tag "Get input from minibuffer" minibuffer))
   :group 'switch-window)
 
 (defcustom switch-window-minibuffer-shortcut nil
@@ -303,7 +305,7 @@ take whatever the last short is.  If a character is specified
 it will always use that key for the minibuffer shortcut.
 
 Note: this feature only works when the value
-of `switch-window-input-style' is 'default ."
+of `switch-window-input-style' is \\='default ."
   :type '(choice (const :tag "Off" nil)
                  (character "m"))
   :group 'switch-window)
@@ -312,16 +314,24 @@ of `switch-window-input-style' is 'default ."
   "Auto resize window's size when switch to a window.
 1. If its value is t, auto resize the selected window.
 2. If its value is a function without arguments,
-   when the returned value it non-nil, auto resize
+   when the returned value is non-nil, auto resize
    the selected window."
+  :type '(choice boolean function)
   :group 'switch-window)
 
 (defcustom switch-window-default-window-size 0.7
   "The default auto resize window's size.
 1. If its value is nil, disable auto resize feature.
-2. If its value is a number (0<x<1), resize selected window to % of frame size.
+2. If its value is a number (0<x<1), resize selected window
+   to fraction of frame size.
 3. If its value is a number (0<x<1) cons, resize selected window to
    car% of frame width and cdr% of frame height."
+  :type '(choice (const :tag "Off" nil)
+                 (float :tag "Fraction of frame size")
+                 (cons
+                  :tag "Fractions of frame width and height"
+                  (float :tag "Fraction of frame width")
+                  (float :tag "Fraction of frame height")))
   :group 'switch-window)
 
 (defcustom switch-window-finish-hook nil
@@ -332,9 +342,10 @@ Its hook function have no arguments."
 
 (defcustom switch-window-preferred 'default
   "Prefer default commands or helm/ivy style commands."
-  :type '(choice (const :tag "Emacs default" 'default)
-                 (const :tag "Helm" 'helm)
-                 (const :tag "Ivy or Counsel" 'ivy))
+  :type '(choice (const :tag "Emacs default" default)
+                 (const :tag "Helm" helm)
+                 (const :tag "Ivy or Counsel" ivy)
+                 (const :tag "Ido" ido))
   :group 'switch-window)
 
 (defvar switch-window-preferred-alist
@@ -343,7 +354,11 @@ Its hook function have no arguments."
      (switch-to-buffer . helm-mini))
     (ivy
      (find-file . counsel-find-file)
-     (switch-to-buffer . ivy-switch-buffer)))
+     (switch-to-buffer . ivy-switch-buffer))
+    (ido
+     (find-file . ido-find-file)
+     (switch-to-buffer . ido-switch-buffer)
+     (dired . ido-dired)))
   "The settings of `switch-window-preferred'.")
 
 (defvar switch-window-extra-map
@@ -390,7 +405,7 @@ This function is used when `switch-window-multiple-frames' is non-nil."
 
 (defface switch-window-background
   '((t (:foreground "gray40")))
-  "Face for switch-window background.")
+  "Face for `switch-window' background.")
 
 (defun switch-window--other-window-or-frame ()
   "If `switch-window-multiple-frames' is set cycle through all visible
@@ -480,7 +495,7 @@ It will start at top left unless FROM-CURRENT-WINDOW is not nil"
                           label (buffer-name (window-buffer win)))))
          (background (switch-window--window-substring win)))
     (funcall switch-window-label-buffer-function win buffer label background)
-    (set-window-buffer win buffer)
+    (set-window-buffer win buffer switch-window-background)
     buffer))
 
 (defun switch-window--window-substring (window)
@@ -497,30 +512,45 @@ It will start at top left unless FROM-CURRENT-WINDOW is not nil"
                    (forward-line height)
                    (point))))
           (concat (propertize
-                   (buffer-substring a b)
+                   (buffer-substring-no-properties a b)
                    'face 'switch-window-background)
                   (buffer-substring b c)
                   (propertize
-                   (buffer-substring c d)
+                   (buffer-substring-no-properties c d)
                    'face 'switch-window-background)))))))
 
-(defun switch-window--create-label-buffer (&optional window buffer label background)
+(defun switch-window--create-label-buffer (&optional _window buffer label background)
   "The default LABEL BUFFER create funcion."
   (with-current-buffer buffer
+    (setq truncate-lines t)
     (when switch-window-background
       (insert background)
       (goto-char (point-min)))
     (cond
      ((eq switch-window-shortcut-appearance 'asciiart)
       (setq line-spacing nil)
-      (insert
-       (replace-regexp-in-string
-        "^\n" ""
-        (nth (cl-position
-              label
-              (remove "" (split-string "123456789abcdefjhijklmnopqrstuvwxyz" ""))
-              :test #'equal)
-             switch-window-asciiart))))
+      (let* ((lines (split-string
+                     (replace-regexp-in-string
+                      "^\n" ""
+                      (nth (cl-position
+                            label
+                            (remove "" (split-string "123456789abcdefghijklmnopqrstuvwxyz;" ""))
+                            :test #'equal)
+                           switch-window-asciiart))
+                     "\n"))
+             (num (apply #'max (mapcar #'length lines))))
+        ;; Deal with "let: End of buffer" problem.
+        ;; Maybe not beautiful, but simple :-)
+        (goto-char (point-max))
+        (dotimes (_ 20)
+          (insert (make-string num ? ))
+          (insert "\n"))
+        (goto-char (point-min))
+        (dolist (line lines)
+          (delete-char num)
+          (insert (format (concat "%-" (number-to-string num) "s") line))
+          (insert "  ")
+          (forward-line 1))))
      ((eq switch-window-shortcut-appearance 'text)
       (insert (propertize label 'face 'switch-window-label)))
      ((eq switch-window-shortcut-appearance 'image)
@@ -640,20 +670,31 @@ TODO: Argument ARG ."
    #'split-window-right arg 1))
 
 ;;;###autoload
-(defun switch-window-then-swap-buffer (arg)
-  "Select a window then swap it buffer with current window's buffer.
-TODO: Argument ARG."
+(defun switch-window-then-swap-buffer (&optional keep-focus)
+  "Swap the current window's buffer with a selected window's buffer.
+
+Move the focus on the newly selected window unless KEEP-FOCUS is
+non-nil (aka keep the focus on the current window).
+
+When a window is strongly dedicated to its buffer, this function
+won't take effect, and no buffers will be switched."
   (interactive "P")
   (let ((buffer1 (window-buffer))
         (window1 (get-buffer-window))
         buffer2 window2)
-    (switch-window)
-    (setq buffer2 (current-buffer))
-    (setq window2 (get-buffer-window))
-    (set-window-buffer window2 buffer1)
-    (set-window-buffer window1 buffer2)
-    (if arg
-        (switch-window--select-window window1))))
+    (if (window-dedicated-p window1)
+        (message "The current window has a dedicated buffer: `%s'" (buffer-name buffer1))
+      (switch-window)
+      (setq buffer2 (current-buffer))
+      (setq window2 (get-buffer-window))
+      (if (window-dedicated-p window2)
+          (progn
+            (select-window window1)
+            (message "The selected window has a dedicated buffer: `%s'" (buffer-name buffer2)))
+        (set-window-buffer window2 buffer1 t)
+        (set-window-buffer window1 buffer2 t)
+        (if keep-focus
+            (switch-window--select-window window1))))))
 
 ;;;###autoload
 (defun switch-window-then-find-file ()
@@ -862,7 +903,7 @@ a window"
         (minibuffer-num nil)
         (num 1)
         key label-buffers
-        window-buffers window-points dedicated-windows)
+        window-buffers window-margins window-points dedicated-windows)
 
     ;; arrange so that C-g will get back to previous window configuration
     (unwind-protect
@@ -873,6 +914,7 @@ a window"
           ;; then display label buffers in all window.
           (dolist (win (switch-window--list))
             (push (cons win (window-buffer win)) window-buffers)
+            (push (cons win (window-margins win)) window-margins)
             (push (cons win (window-point win)) window-points)
             (when (window-dedicated-p win)
               (push (cons win (window-dedicated-p win)) dedicated-windows)
@@ -882,25 +924,35 @@ a window"
               (push (switch-window--display-number win num) label-buffers))
             (setq num (1+ num)))
           ;; get user's input
-          (cond ((eq switch-window-input-style 'default)
+          (cond ((eq switch-window-input-style 'read-event)
                  (setq key (switch-window--get-input
                             prompt-message minibuffer-num eobps)))
-                ((eq switch-window-input-style 'minibuffer)
+                ((member switch-window-input-style '(default minibuffer))
                  (setq key (switch-window--get-minibuffer-input
                             prompt-message minibuffer-num eobps)))))
       ;; clean input-method-previous-message
       (setq input-method-previous-message nil)
       ;; restore original cursor
       (setq-default cursor-type original-cursor)
-      ;; remove all label-buffers, useless now.
-      (mapc 'kill-buffer label-buffers)
       ;; Restore window's buffer, point and dedicate state.
       (dolist (w window-buffers)
-        (set-window-buffer (car w) (cdr w)))
+        (set-window-buffer (car w) (cdr w) t))
+      ;; Restore window's margins.
+      (dolist (w window-margins)
+        (set-window-margins (car w) (cadr w) (cddr w)))
       (dolist (w window-points)
         (set-window-point (car w) (cdr w)))
       (dolist (w dedicated-windows)
-        (set-window-dedicated-p (car w) (cdr w))))
+        (set-window-dedicated-p (car w) (cdr w)))
+
+      ;; remove all label-buffers, useless now.
+      ;; it's important to remove temporary label buffers after restoring original buffers
+      ;; because for protected windows it will also kill the window via `replace-buffer-in-windows''`
+      ;; according to https://www.gnu.org/software/emacs/manual/html_node/elisp/Killing-Buffers.html
+      ;; for protected window a temporary lable buffer will be the only it displays
+      ;; For example, it will kill Treemacs buffer and it's window
+      ;; thus failing to restore it if you run the killing loop before.
+      (mapc 'kill-buffer label-buffers))
     key))
 
 (define-minor-mode switch-window-mouse-mode
